from typing import Union
from copy import deepcopy
import numpy as np
import pandas as pd
from candlelite.crypto.binace_lite import BINANCE_TIMEZONE
from candlelite.calculate.transform import to_candle as _to_candle
from paux import date as _date


# 将Binance的历史K线数据从array转换成DataFrame类型
def candle_to_df(
        candle: np.ndarray,
        convert_ts: bool = True,
        timezone: Union[str, None] = BINANCE_TIMEZONE,
) -> pd.DataFrame:
    '''
    :param candle: array类型的历史K线数据
    :param convert_ts: 是否将时间戳转化为日期字符串
    :param timezone: 时区
        默认使用candlelite中Binance的时区
        None 使用本机默认时区
    '''
    df = pd.DataFrame(candle)
    df.columns = [
        'openTs',  # 开盘时间 Open time
        'open',  # 开盘价 Open
        'high',  # 最高价 High
        'low',  # 最低价 Low
        'close',  # 收盘价(当前K线未结束的即为最新价) Close
        'volume',  # 成交量 Volume
        'closeTs',  # 收盘时间 Close time
        'turnover',  # 成交额 Quote asset volume
        'tradeNum',  # 成交笔数 Number of trades
        'buyVolume',  # 主动买入成交量 Taker buy base asset volume
        'buyTurnover',  # 主动买入成交额 Taker buy quote asset volume
        'ignore'  # 请忽略该参数 Ignore
    ]
    df = df.drop(columns=['ignore'])
    # 是否转换时间戳为日期字符串
    if convert_ts:
        # 美国时间
        if timezone == 'America/New_York':
            fmt = '%m/%d/%Y %H:%M:%S'
        # 中国时间
        else:
            fmt = '%Y-%m-%d %H:%M:%S'
        df['openTs'] = df['openTs'].apply(
            lambda openTs: _date.to_fmt(date=openTs, timezone=timezone, fmt=fmt)
        )
        df['closeTs'] = df['closeTs'].apply(
            lambda closeTs: _date.to_fmt(date=closeTs, timezone=timezone, fmt=fmt)
        )
    return df


# 将Binance的历史K线数据从DataFrame转换成array类型
def df_to_candle(
        df: pd.DataFrame,
        timezone: Union[str,None]= BINANCE_TIMEZONE
):
    '''
    :param df: DataFrame类型的历史K线数据
    :param timezone: 时区
        默认使用candlelite中Binance的时区
        None 使用本机默认时区
    '''
    df = deepcopy(df)
    df['openTs'] = df['openTs'].apply(
        lambda openTs: _date.to_ts(date=openTs, timezone=timezone)
    )
    df['closeTs'] = df['closeTs'].apply(
        lambda closeTs: _date.to_ts(date=closeTs, timezone=timezone)
    )
    candle = _to_candle(df)
    return candle



from binance_candle import Market
from binance_candle.utils import candle_to_df, df_to_candle
from pprint import pprint

if __name__ == '__main__':
    # 币币交易：SPOT；U本位合约：UM；币本位合约：CM
    instType = 'UM'
    # 实例化行情Market
    market = Market(instType, timezone='America/New_York')
    # 按照起始时间获取历史K线
    result = market.get_history_candle(
        symbol='BTCUSDT',
        start='2023-01-01 00:00:00',
        end='2023-01-01 10:00:00',
        bar='1m',
    )
    candle = result['data']
    # <candle:np.ndarray> 转化为 <df:pd.DataFrame>
    df = candle_to_df(candle)
    pprint(df)
    # <df:pd.DataFrame> 转化为 <candle:np.ndarray>
    candle = df_to_candle(df)
    pprint(candle)



from binance_candle.server.rule import CandleRule
from binance_candle.server.server import CandleServer


class CandleServerSPOT(CandleServer):
    def __init__(self, rule):
        instType = 'SPOT'
        super(CandleServerSPOT, self).__init__(instType=instType, rule=rule)


class CandleServerUM(CandleServer):
    def __init__(self, rule):
        instType = 'UM'
        super(CandleServerUM, self).__init__(instType=instType, rule=rule)


class CandleServerCM(CandleServer):
    def __init__(self, rule):
        instType = 'Cm'
        super(CandleServerCM, self).__init__(instType=instType, rule=rule)



from binance_candle.market import (Market, MarketUM, MarketCM, MarketSPOT)
from binance_candle.server import (CandleRule, CandleServer, CandleServerSPOT, CandleServerUM, CandleServerCM)
from binance_candle import utils
from candlelite.crypto.binace_lite import BinanceLite

__version__ = '1.0.6'



from binance_candle import Market
from pprint import pprint

if __name__ == '__main__':
    # 币币交易：SPOT；U本位合约：UM；币本位合约：CM
    instType = 'SPOT'
    # 实例化行情Market
    market = Market(instType)
    # 全部产品的最新成交价格
    pprint(market.get_tickerPricesMap())  # 字典类型
    pprint(market.get_tickerPrices())  # 列表类型
    # 单个产品的最新成交价格（BTTUSDT）
    pprint(market.get_tickerPrice('BTCUSDT'))



from binance_candle.market._base import MarketBase
from typing import Union

__all__ = ['Ticker']


class Ticker(MarketBase):

    # 获取全部产品的最优挂单
    # Weight: 现货 2 合约 5
    def get_bookTickers(self) -> dict:
        '''
        现货
            https://binance-docs.github.io/apidocs/spot/cn/#5393cd07b4
        U本位合约
            https://binance-docs.github.io/apidocs/futures/cn/#5393cd07b4
        币本位合约
            https://binance-docs.github.io/apidocs/delivery/cn/#5393cd07b4
        '''
        return self.inst.market.get_ticker_bookTicker()

    # 获取全部产品的最优挂单 （字典格式）
    # Weight: 现货 2 合约 5
    def get_bookTickersMap(self) -> dict:
        get_bookTickers_result = self.get_bookTickers()
        # [ERROR RETURN] 数据异常
        if get_bookTickers_result['code'] != 200:
            return get_bookTickers_result
        data_map = {}
        for ticker in get_bookTickers_result['data']:
            symbol = ticker['symbol']
            data_map[symbol] = ticker

        get_bookTickers_result['data'] = data_map
        return get_bookTickers_result

    # 获取一个产品的最优挂单
    # Weight: 现货 1 合约 2
    def get_bookTicker(self, symbol: str) -> dict:
        '''
        现货
            https://binance-docs.github.io/apidocs/spot/cn/#5393cd07b4
        U本位合约
            https://binance-docs.github.io/apidocs/futures/cn/#5393cd07b4
        币本位合约
            https://binance-docs.github.io/apidocs/delivery/cn/#5393cd07b4
        :param symbol: 产品
        '''
        return self.inst.market.get_ticker_bookTicker(symbol=symbol)

    # 获取全部产品的最新价格
    # Weight: 2
    def get_tickerPrices(self):
        '''
        现货
            https://binance-docs.github.io/apidocs/spot/cn/#8ff46b58de
        U本位合约
            https://binance-docs.github.io/apidocs/futures/cn/#8ff46b58de
        币本位合约
            https://binance-docs.github.io/apidocs/delivery/cn/#8ff46b58de
        '''
        return self.inst.market.get_ticker_price(symbol='')

    # 获取全部产品的最新价格 (字典格式)
    # Weight: 2
    def get_tickerPricesMap(self):
        get_tickerPrices_result = self.get_tickerPrices()
        # [ERROR RETURN] 数据异常
        if get_tickerPrices_result['code'] != 200:
            return get_tickerPrices_result
        data_map = {}
        for ticker in get_tickerPrices_result['data']:
            symbol = ticker['symbol']
            data_map[symbol] = ticker

        get_tickerPrices_result['data'] = data_map
        return get_tickerPrices_result

    # 获取一个产品的最新价格
    # Weight: 1
    def get_tickerPrice(self, symbol: str):
        '''
        现货
            https://binance-docs.github.io/apidocs/spot/cn/#8ff46b58de
        U本位合约
            https://binance-docs.github.io/apidocs/futures/cn/#8ff46b58de
        币本位合约
            https://binance-docs.github.io/apidocs/delivery/cn/#8ff46b58de
        :param symbol: 产品
        '''
        return self.inst.market.get_ticker_price(symbol=symbol)

    def get_depth(self, symbol: str, limit: Union[int, str] = ''):
        '''
        现货
            https://binance-docs.github.io/apidocs/spot/cn/#38a975b802
        U本位合约
            https://binance-docs.github.io/apidocs/futures/cn/#38a975b802
        币本位合约
            https://binance-docs.github.io/apidocs/delivery/cn/#38a975b802
        :param symbol: 产品
        :param limit: 数量
        '''
        return self.inst.market.get_depth(symbol=symbol, limit=limit)



from binance_candle.market.history_candle import HistoryCandle  # 历史K线
from binance_candle.market.exchange_info import ExchangeInfo  # 交易对信息
from binance_candle.market.ticker import Ticker  # 实时价格

__all__ = ['Market', 'MarketSPOT', 'MarketUM', 'MarketCM']


class Market(HistoryCandle, Ticker, ExchangeInfo):
    pass


class MarketSPOT(Market):
    def __init__(self, key: str = '', secret: str = '', timezone: str = 'America/New_York'):
        instType = 'SPOT'
        super(MarketSPOT, self).__init__(instType=instType, key=key, secret=secret, timezone=timezone)


class MarketUM(Market):
    def __init__(self, key: str = '', secret: str = '', timezone: str = 'America/New_York'):
        instType = 'UM'
        super(MarketUM, self).__init__(instType=instType, key=key, secret=secret, timezone=timezone)


class MarketCM(Market):
    def __init__(self, key: str = '', secret: str = '', timezone: str = 'America/New_York'):
        instType = 'CM'
        super(MarketCM, self).__init__(instType=instType, key=key, secret=secret, timezone=timezone)



from typing import Literal
import pbinance
from binance_candle import exception


# 市场信息基类
class MarketBase(pbinance.Binance):

    def __init__(
            self,
            instType: Literal['SPOT', 'UM', 'CM'],
            key: str = '',
            secret: str = '',
            timezone='America/New_York'
    ):
        '''
        主要为了处理inst
            instType = 'SPOT'   self.inst -> self.spot
            instType = 'UM'     self.inst -> self.um
            instType = 'CM'     self.inst -> self.cm

        :param instType: 产品类型
            SPOT:   现货
            UM:     U本位合约
            CM:     币本位合约
        :param key: API KEY 可以不填写
        :param secret: API SECRET 可以不填写
        :param timezone: 时区
            America/New_York:   美国时间
            Asia/Shanghai:      中国时间
        '''
        # 父类
        super(MarketBase, self).__init__(key=key, secret=secret)
        # 时区与产品类别
        self.timezone = timezone
        self.instType = instType.upper()
        # inst
        if self.instType == 'SPOT':  # 现货交易
            self.inst = self.spot
        elif self.instType == 'UM':  # U本位
            self.inst = self.um
        elif self.instType == 'CM':  # 币本位
            self.inst = self.cm
        else:
            msg = '[instType error] your input instType={instType} instType must in ["SPOT","UM","CM","EO"]'.format(
                instType=str(instType)
            )
            raise exception.ParamException(msg)



from binance_candle import Market
from pprint import pprint

if __name__ == '__main__':
    # 币币交易：SPOT；U本位合约：UM；币本位合约：CM
    instType = 'UM'
    # 实例化行情Market
    market = Market(instType)
    # 单个产品的深度信息 limit : 数量
    pprint(market.get_depth('BTCUSDT', limit=10))



from binance_candle.exception._base import AbstractEXP


# 请求状态码异常
class CodeException(AbstractEXP):
    def __init__(self, msg):
        self.error_msg = msg


# 请求异常
class RequestException(AbstractEXP):
    def __init__(self, msg):
        self.error_msg = msg



from binance_candle import Market
from pprint import pprint

if __name__ == '__main__':
    # 币币交易：SPOT；U本位合约：UM；币本位合约：CM
    instType = 'SPOT'
    # 实例化行情Market
    market = Market(instType)
    # 产品类型的全部交易规范
    exchangeInfos = market.get_exchangeInfos()
    pprint(exchangeInfos)
    # BTCUSDT的交易规范
    BTCUSDT_enchangeInfo = market.get_exchangeInfo('BTCUSDT')
    pprint(BTCUSDT_enchangeInfo)



from binance_candle import Market
from pprint import pprint

if __name__ == '__main__':
    # 币币交易：SPOT；U本位合约：UM；币本位合约：CM
    instType = 'SPOT'
    # 实例化行情Market
    market = Market(instType)
    # 全部产品的最优挂单
    pprint(market.get_bookTickersMap())  # 字典类型
    pprint(market.get_bookTickers())  # 列表类型
    # 单个产品的最优挂单（BTTUSDT）
    pprint(market.get_bookTicker('BTCUSDT'))

